#ifndef AGENT_H
#define AGENT_H

#include <iostream>
#include <cstdlib>
#include "Strategy.h"
using namespace std;

//! \brief Agent Class.
/*!
* This is the Agent class which contains the Agent member functions.
* The constructor intializes an individual Agent Object while the other member functions
* 
* take from a pool of 20 best stragies at random
*************************************************************************************/
class Agent {

public:

	//! \brief Default Agent Constructor
	/*!
	* Default constructor NOT utilized within code.
	***************************************************************/
	Agent();
	
	/*! \brief Agent Destructor
	*	
	* Destructor deletes an Agents Strategies when destructing the Angent object.
	*******************************************************************************/
	~Agent();
	
	//! \brief Agent Reborn Constructor
	/*!
	* \details Description: Initializes an Agents Strategy.
	* \param numberOfBars indicates how many bars will be in the simulation. Initializes an Agents STM
	* \param avgAge Indicates the Average Age at which an Agent will expire.
	* \param dropS Indicates the threshold at which the given Agent will replace the poorly performing Strategy
	* \param s1 One of the 20 best performing Strategy's that this Agent will have in their STM
	* \param s2 One of the 20 best performing Strategy's that this Agent will have in their STM
	* \param s3 One of the 20 best performing Strategy's that this Agent will have in their STM
	* \pre 0 \< numberofBars \< 256, 0 \< avgAge \< 100, 0 \< dropS \< 100, The Simulation has begun. 
	* \post An Agents Strategies has been initialized to random values .
	********************************************************************/
	Agent(int numberOfBars, int avgAge, int dropS, Strategy* s1, Strategy* s2, Strategy *s3);
	
	//! \brief Agent Constructor
	/*!
	* \details Description: Initializes an Agents Strategy.
	* \param numberOfBars indicates how many bars will be in the simulation. Initializes an Agents STM
	* \param avgAge Indicates the Average Age at which an Agent will expire.
	* \param dropS Indicates the threshold at which the given Agent will replace the poorly performing Strategy
	* \pre 0 \< numberofBars \< 256, 0 \< avgAge \< 100, 0 \< dropS \< 100, The Simulation has begun. 
	* \post An Agent Object has been created as well as its Strategies has been initialized to random values.
	********************************************************************/
	Agent(int numberOfBars, int avgAge, int dropS);
	
	//! \brief Computational function which decides which Bar to visit
	/*!
	* \details Description: This function, using a random number generator, 
	* picks a strategy (i.e. Strategy 0 to 2)  to utilize where the Agent is 
	* going to bar for the current turn.
	* \param STM Integer pointer that refers to the STM structure. \see Town::STM, Town
	* \pre A turn has just begun.
	* \post The Agent successfully decided where they are going for the current turn.
	* \return Integer indicating which Bar Object the Agent has decided to visit for the
	* current round.
	********************************************************************/
	int isGoingToBar(int STM);
	
	//! \brief Agent attribute Updater
	/*!
	* \details Description: This function takes in an array of currents turn bar winners and the current STM, 
	* it then uses this information to update Strategy scores and if score drops below predefined threshold
	* the strategy is replaced with another. It then returns needed variables.
	* to the GUI.
	* \param winners[] holds 1 if bar won 0 if bar lost
	* \param STM Short Term Memory
	* \pre Bars have decided if they won the current round.
	* \post Strategies score of agent has been updated appropriately and strategies have been replaced of necessary.
	* \return Double pointer referencing maxScore and avgScore
	********************************************************************/
	Strategy** tellWins(int winners[],int STM);
	
	//! \brief Determines if an Agent has expired.
	/*!
	* \details Description: This function references an Agents protected variable, "age" and compares this value to 
	* an Agents predetermined age to die named "death". The function returns the outcome of this comparision.
	* \pre An Agent object has been fully initialized.
	* \param stuff Pointer holding data for graphs. 
	* \post None.
	* \return Boolean of either True or False.
	****************************************************/
	bool isDead(pass_graph* stuff);

protected:

	/*! \brief Strategy Container
	*
	* Contains the three Strategy options available to an Agent.
	*/
	Strategy** strats;
	
	/*! \brief Number Of Bars
	*
	* Indicates the number of Bar Objects in the simulation.
	*/
	int numberOfBars;
	
	/*! \brief An Agent's current age
	*
	* Indicates how old an Agent is.
	*/
	int age;
	
	/*! \brief An Agents predetermined expiration date.
	*
	* Indicates the number of rounds an Agent will participate in before expiring 
	* and no longer particapting for the rest of the duration of the simulation.
	* \note Set to -1 if death option is NOT selected.
	*/
	int death;
	
	/*! \brief Attended Bar
	*
	* Indicates the Bar number that the Agent has attended for a current round.
	* \note NOT modifiable. For score keeping purposes only.
	*/
	int bar;
	
	/*! \brief Alpha Value
	*
	* Indicates the threshold for which a strategy will be dropped when an agent is in a group.
	* \note NOT modifiable. For score keeping purposes only.
	*/
	int drop;
};

#endif


