#ifndef GROUP_H
#define GROUP_H
#include "Agent.h"

#include <iostream>
#include <cstdlib>

//! \brief Group Class.
/*!
* This is the Group class which contains the new needed member functions which make the 
* group option in our Simulation Application be available to be selected as an additional
* "Population Variable" by the user to affect the simulation with "Group Behavior"
* 
*************************************************************************************/
class Group {

public:

	//! \brief Default Agent Constructor
	/*!
	* Default constructor NOT utilized within code.
	***************************************************************/
	Group();
	
	/*! \brief Agent Destructor
	*
	* Deletes a group, when the set of ALL Agents within the group is NULL.
	*******************************************************************************/
	~Group();
	
	//! \brief Group Constructor (creates group with Agents assigned)
	/*!
	* \details Description: Initializes a Group within a Simulation profile.
	* \param nump number of people going to be in the group
	* \param numb number of bars in the simulation
	* \param avgAge average death age of Agents within this group
	* \param drop the threshold value at which an Agent drops a strategy
	* \pre 0 \< nump \< 2*numOfBars, 0 \< numb \< 256 , 0 \< avgAge \< 100, 0 \< drop \< 100, 
	* A Town ohject as well as all Agent 
	* and Bar ojects have been previously initialized. There is a limit of: numberOfAgents / (2*numberOfBars)
	* initialized groups within the simulation. No more will be created beyond this number.
	* \post One group within the Agent population has been assigned to a certain number of Agents.
	********************************************************************/
	Group(int nump, int numb,int avgAge, int drop);
	
	//! \brief Group Constructor (No Agents initially Assigned to group)
	/*!
	* \details Description: Initializes a Group within a Simulation profile.
	* \param nump number of people going to be in the graph
	* \param numb number of bars in the simulation
	* \pre 0 \< nump \< 2*numOfBars, 0 \< numb \< 256 , 0 \< avgAge \< 100, 0 \< drop \< 100, 
	* A Town ohject as well as all Agent 
	* and Bar ojects have been previously initialized. There is a limit of: numberOfAgents / (2*numberOfBars)
	* initialized groups within the simulation. No more will be created beyond this number.
	* \post An empty group has been created.
	********************************************************************/
	Group(int nump, int numb);
	
	
	//! \brief Groups decision of which Bar to visit for a given round.
	/*! 
	* \details Description: Variation of an Agent's isGoingToBar() function. This function takes a 
	* consensus of all agents within the given group and decides to which Bar the given group would attend.
	* \param STI pointer to short term index of the combined Agents best scoring strategy's. If there is a 
	* tie, it is broken by comparing which bar has resulting in Agents having a better time for past turns.
	* If that factor is also a tie then it is randomly assigned.
	* \pre A turn has just begun.
	* \post A Bar to visit for the given turn has been decided.
	* \return The Bar number to visit the specified group will visit for the given turn.
	* \note This function is based on the assumption that friends of a group would always want to stick 
	* together when they go to a bar.
	* \see See also: Agent::isGoingToBar(int)
	***********************************************************************/
	int isGoingToBar(int STI);
	
	//! \brief Number of people in a griven group
	/*!
	* \details Description: Returns an integer representing the number of people currently within a group for 
	* any given time during a simulation.
	* \pre A group has been initialized
	* \post The number of people in the given group has been returned.
	* \return An integer representing the number of Agents within the specified group.
	************************************************************************/
	int getNumPeeps();
	
	//! \brief Adding a new person to an existing group.
	/*!
	* \brief Description: This function adds a newly created person to an existing group during the propagation of 
	* an ongoing simulation, 
	* \param avgAge average death age of Agents within this group
	* \param drop the threshold value at which an Agent drops a strategy
	* \pre A new Agent instance has just been created and at least one group within the population is not at
	* full capacity.
	* \post The newly created Agent has been assigned to a group
	* \note This function is called at any time a new Agent is created.
	***************************************************************************/
	void addAgent(int avgAge, int drop);
	
	//! \brief Adding a new person to an existing group.
	/*!
	* \brief Description: This function adds a newly created person to an existing group during the propagation of 
	* an ongoing simulation, 
	* \param f A reference to an Agent object.
	* \pre At least 1 Agent instance exists and at least one group within the population is not at
	* full capacity.
	* \post An Agent that already exists within the population has been assigned to a group
	* \note Used to copy an already existing Agent in the population.
	***************************************************************************/
	void addAgent(Agent* f);
	
	
	//! \brief Determines if a group is empty
	/*!
	* \details Description: This function returns a boolean of whether or not the specified group is or is NOT empy.
	* Where empty represents a NULL set of Agents.
	* \param g Pointer to integer that keeps track of dead people.
	* \param stuff Pointer to structure holding data for graphs
	* \pre The given group has been initialized.
	* \post None.
	* \return A boolean representing whether the specified group contains a NULL set of Agents.
	*****************************************************************************/
	bool isEmpty(int *g, pass_graph* stuff);
	
	//! \brief Group attribute Updater
	/*!
	* \details Description: This function takes in an array of the current turn bar winners and the current STM, 
	* it then uses this information to update Strategy scores (in this case the groups) and if score drops below predefined threshold
	* the strategy is replaced with another. It then returns needed variables.
	* to the GUI.
	* \param winners holds 1 if bar won 0 if bar lost
	* \param STM Short Term Memory
	* \param g TEMP
	* \pre Bars have decided if they won the current round.
	* \post Strategies score of agent has been updated appropriately and strategies have been replaced of necessary.
	* \return Strategy pointer referencing maxScore and avgScore
	********************************************************************/
	Strategy ** Group::tellWins(int* winners,int STM,int* g);

protected:

	/*! \brief Agent Holding Structure
	*
	* This structure contains all the Agents currently in the group.
	*/
	Agent** peeps;
	
	/*! \brief Number of Agents within Group.
	*
	* Indicates the number of Agents currently within the group.
	*/
	int numPeeps;
	
	/*! \brief Number Of Bars
	*
	* Indicates how many bars are present in the Simulation
	*/
	int numBars;
	
	/*! \brief Groups LTM
	*
	* A pointer containing the long term memory which stores how many times an Agent has had a good 
	* time at certain bar so far in the simulation.
	* \note This attribute is utilized in isGoingToBar() 's secondary decision of how many times
	* each Agent has had a good time at each proposed bar.
	*/
	int * LTM;
	
	/*! \brief Average Age
	*
	* Indicates what the Gaussian Mean should be for the entire Simulation profile.
	*/
	int avgAge;
};
#endif


